package com.ruoyi.zhdd.service;

import com.ruoyi.zhdd.domain.ChemicalData;
import com.ruoyi.zhdd.domain.vo.ChemicalDataVo;
import com.ruoyi.zhdd.domain.bo.ChemicalDataBo;
import com.ruoyi.common.core.mybatisplus.core.IServicePlus;
import com.ruoyi.common.core.page.TableDataInfo;

import java.util.Collection;
import java.util.List;

/**
 * 危化品信息Service接口
 *
 * @author xintong
 * @date 2022-01-14
 */
public interface IChemicalDataService extends IServicePlus<ChemicalData, ChemicalDataVo> {
	/**
	 * 查询单个
	 * @return
	 */
	ChemicalDataVo queryById(String id);

	/**
	 * 查询列表
	 */
    TableDataInfo<ChemicalDataVo> queryPageList(ChemicalDataBo bo);

	/**
	 * 查询列表
	 */
	List<ChemicalDataVo> queryList(ChemicalDataBo bo);

	/**
	 * 根据新增业务对象插入危化品信息
	 * @param bo 危化品信息新增业务对象
	 * @return
	 */
	Boolean insertByBo(ChemicalDataBo bo);

	/**
	 * 根据编辑业务对象修改危化品信息
	 * @param bo 危化品信息编辑业务对象
	 * @return
	 */
	Boolean updateByBo(ChemicalDataBo bo);

	/**
	 * 校验并删除数据
	 * @param ids 主键集合
	 * @param isValid 是否校验,true-删除前校验,false-不校验
	 * @return
	 */
	Boolean deleteWithValidByIds(Collection<String> ids, Boolean isValid);
}
